import click
import requests
from redash_toolbelt.client import Redash

template = u"""/*
Name: {name}
Data source: {data_source}
Created By: {created_by}
Last Updated At: {last_updated_at}
*/
{query}"""


def save_queries(queries):
    for query in queries:
        filename = "query_{}.sql".format(query["id"])
        with open(filename, "w") as f:
            content = template.format(
                name=query["name"],
                data_source=query["data_source_id"],
                created_by=query["user"]["name"],
                last_updated_at=query["updated_at"],
                query=query["query"],
            )
            f.write(content)


@click.command()
@click.argument("redash_url")
@click.option(
    "--api-key",
    "api_key",
    required=True,
    envvar="REDASH_API_KEY",
    show_envvar=True,
    prompt="API Key",
    help="User API Key",
)
def main(redash_url, api_key):
    redash = Redash(redash_url, api_key)
    queries = redash.paginate(redash.queries)
    save_queries(queries)


if __name__ == "__main__":
    main()



import itertools
import json
import re

import click
import pytest

from redash_toolbelt import Redash


def find_table_names(url, key, data_source_id):

    client = Redash(url, key)

    schema_tables = [
        token.get("name")
        for token in client._get(f"api/data_sources/{data_source_id}/schema")
        .json()
        .get("schema", [])
    ]

    queries = [
        query
        for query in client.paginate(client.queries)
        if query.get("data_source_id", None) == int(data_source_id)
    ]

    tables_by_qry = {
        query["id"]: [
            table
            for table in extract_table_names(query["query"])
            if table in schema_tables or len(schema_tables) == 0
        ]
        for query in queries
    }

    return tables_by_qry


def format_query(str_sql):
    """Strips all newlines, excess whitespace, and spaces around commas"""

    stage1 = str_sql.replace("\n", " ")
    stage2 = re.sub(r"\s+", " ", stage1).strip()
    stage3 = re.sub(r"(\s*,\s*)", ",", stage2)

    return stage3


def extract_table_names(str_sql):

    fmt_sql = format_query(str_sql)

    # This pattern captures all text that immdiately follows a FROM or JOIN
    # keyword except whitespace and parentheses. It effectively removes table
    # aliases like "<table> as t" or "<table> t"
    PATTERN = re.compile(
        r"(?:FROM|JOIN)(?:\s+)([^\s\(\)]+)", flags=re.IGNORECASE | re.UNICODE
    )

    regex_matches = [match for match in re.findall(PATTERN, fmt_sql)]

    # For test_6: expand any comma-delimitted matches
    split_matches = [i.split(",") for i in regex_matches]
    flattened_split_matches = [i for i in itertools.chain(*split_matches)]

    # For test_8 and test_9: expand aliased comma-delimitted matches
    #   1. Find all text between FROM->WHERE and FROM->JOIN keywords
    #   2. Replace any commas with `FROM` keywords
    #   3. Apply the same PATTERN match used above which ignores aliases
    after_from_before_keyword = [
        match for match in re.findall(r"(?:FROM)(.*)(?:WHERE|JOIN)", fmt_sql)
    ]
    sub_keyword_for_comma = [
        segment.replace(",", " FROM ") for segment in after_from_before_keyword
    ]
    sub_regex_matches = [
        re.findall(PATTERN, format_query(i)) for i in sub_keyword_for_comma
    ]

    flattened_sub_matches = [
        i
        for i in itertools.chain(*sub_regex_matches)
        if i not in flattened_split_matches
    ]

    return [*flattened_split_matches, *flattened_sub_matches]


def print_summary(tables_by_qry):
    """Builds a summary showing table names and count of queries that reference them."""

    summary = {
        table_name: sum(
            [1 for tables in tables_by_qry.values() if table_name in tables]
        )
        for table_name in itertools.chain(*tables_by_qry.values())
    }

    align = max([len(table_name) for table_name in summary.keys()])

    print("\n")
    print(f"{'table':>{align}} | {'number of queries':>17}")
    print("-" * align + " | " + "-" * 17)

    for t, num in sorted(summary.items(), key=lambda item: item[1], reverse=True):
        print(f"{t:>{align}} | {num:>17}")

    print("\n")


def print_details(tables_by_qry):
    """Prints out (query_id, tablename) tuples"""

    details = [
        [(query, table) for table in tables] for query, tables in tables_by_qry.items()
    ]

    for row in itertools.chain(*details):
        print(",".join([str(i) for i in row]))


@click.command()
@click.argument("url",)
@click.argument("key",)
@click.argument("data_source_id")
@click.option("--detail", is_flag=True, help="Prints out all table/query pairs?")
def main(url, key, data_source_id, detail):
    """Find table names referenced in queries against DATA_SOURCE_ID"""

    data = find_table_names(url, key, data_source_id)

    if detail:
        print_details(data)
    else:
        print_summary(data)


if __name__ == "__main__":
    main()


# ┌───────────────────────────────────────────┐
# │                                           │
# │      ________           _____             │
# │      ___  __/_____________  /________     │
# │      __  /  _  _ \_  ___/  __/_  ___/     │
# │      _  /   /  __/(__  )/ /_ _(__  )      │
# │      /_/    \___//____/ \__/ /____/       │
# │                                           │
# │                                           │
# └───────────────────────────────────────────┘


def test_1():

    sql = """
    SELECT field FROM table0 LEFT JOIN table1 ON table0.field = table1.field
    """

    tables = extract_table_names(sql)
    expected = ["table0", "table1"]

    assert len(tables) == len(expected) and all([i in expected for i in tables])


def test_2():

    sql = """
    SELECT field FROM table0 as a LEFT JOIN table1 as b ON a.field = b.field
    """

    tables = extract_table_names(sql)
    expected = ["table0", "table1"]

    assert len(tables) == len(expected) and all([i in expected for i in tables])


def test_3():

    sql = """
    SELECT field FROM table0 a LEFT JOIN table1 b ON a.field = b.field
    """

    tables = extract_table_names(sql)
    expected = ["table0", "table1"]

    assert len(tables) == len(expected) and all([i in expected for i in tables])


def test_4():

    sql = """
    SELECT field FROM schema.table0 a LEFT JOIN schema.table1 b ON a.field = b.field
    """

    tables = extract_table_names(sql)
    expected = ["schema.table0", "schema.table1"]

    assert len(tables) == len(expected) and all([i in expected for i in tables])


def test_5():

    sql = """
    SELECT field
    FROM
        table0
    LEFT JOIN
        table1
    """

    tables = extract_table_names(sql)
    expected = ["table0", "table1"]

    assert len(tables) == len(expected) and all([i in expected for i in tables])


def test_6():

    sql = """
    SELECT field FROM table1,table2, table3 ,table4 , table5
            , table6
    WHERE table0.field = table1.field
    """

    tables = extract_table_names(sql)
    expected = ["table1", "table2", "table3", "table4", "table5", "table6"]

    assert len(tables) == len(expected) and all([i in expected for i in tables])


def test_7():

    sql = """
    SELECT field FROM [table0] LEFT JOIN [table1] ON [table0].field = [table1].field
    """

    tables = extract_table_names(sql)
    expected = ["[table0]", "[table1]"]

    assert len(tables) == len(expected) and all([i in expected for i in tables])


def test_8():

    sql = """
    SELECT field FROM table1 AS t1, table2 AS t2, table3 t3, table4 t4 WHERE t1.field = 'value'
    """

    tables = extract_table_names(sql)
    expected = ["table1", "table2", "table3", "table4"]

    assert len(tables) == len(expected) and all([i in expected for i in tables])


def test_9():

    sql = """
    SELECT field FROM table1 AS t1, table2 AS t2, table3 t3, table4 t4
    LEFT JOIN table5 ON t1.field = table5.field
    WHERE t1.field = t2.field AND t2.field = t3.field AND t3.field=t4.field
    """

    tables = extract_table_names(sql)
    expected = ["table1", "table2", "table3", "table4", "table5"]

    assert len(tables) == len(expected) and all([i in expected for i in tables])



import click

from redash_toolbelt import Redash


class Lookup(object):
    def __init__(self, redash, email_list):
        self.email_list = [i.lower() for i in email_list]
        self.redash = redash

    def check_query_result(self, query_result_id):
        if not query_result_id:
            return False

        result = self.redash._get("api/query_results/{}".format(query_result_id))

        return any([email in result.text.lower() for email in self.email_list])

    def check_query(self, query):

        found_in_query = any(
            [
                email in (query[field] or "").lower()
                for email in self.email_list
                for field in ["query", "name", "description"]
            ]
        )

        found_in_tags = any(
            [
                email in (tag or "").lower()
                for email in self.email_list
                for tag in query["tags"]
            ]
        )

        found_in_result = self.check_query_result(query["latest_query_data_id"])

        return found_in_query or found_in_result or found_in_tags

    def check_dashboard(self, dashboard):

        found_in_dashboard = any(
            [
                email in (dashboard[field] or "").lower()
                for email in self.email_list
                for field in ("slug", "name")
            ]
        )
        found_in_tags = any(
            [
                email in (tag or "").lower()
                for email in self.email_list
                for tag in dashboard["tags"]
            ]
        )

        if found_in_dashboard or found_in_tags:
            return True

        dash_widgets = (
            self.redash._get("api/dashboards/{}".format(dashboard["slug"]))
            .json()
            .get("widgets", [])
        )

        # Check text widgets
        found_in_widget = any(
            [
                email in widget["text"]
                for email in self.email_list
                for widget in dash_widgets
                if "visualization" not in widget
            ]
        )

        return found_in_dashboard or found_in_widget or found_in_tags

    def lookup(self):
        queries = self.redash.paginate(self.redash.queries)

        with click.progressbar(queries, label="Queries") as bar:
            found_q = [query for query in bar if self.check_query(query)]

        for query in found_q:
            query_url = "{}/queries/{}".format(self.redash.redash_url, query["id"])
            print(query_url)

        dashboards = self.redash.paginate(self.redash.dashboards)

        with click.progressbar(dashboards, label="Dashboards") as bar:
            found_d = [dash for dash in bar if self.check_dashboard(dash)]

        for dash in found_d:
            dash_url = "{}/dashboards/{}".format(self.redash.redash_url, dash["slug"])
            print(dash_url)


@click.command()
@click.argument("redash_host")
@click.argument("email", nargs=-1)
@click.option(
    "--api-key",
    "api_key",
    envvar="REDASH_API_KEY",
    show_envvar=True,
    prompt="API Key",
    help="User API Key",
)
def lookup(redash_host, email, api_key):
    """Search for EMAIL in queries and query results, output query URL if found."""

    redash = Redash(redash_host, api_key)
    lookup = Lookup(redash, email)
    lookup.lookup()


if __name__ == "__main__":
    lookup()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['redash_toolbelt', 'redash_toolbelt.examples']

package_data = \
{'': ['*'],
 'redash_toolbelt.examples': ['20210302_adventureclub_query_export/*']}

install_requires = \
['click>=7.0,<8.0', 'requests>=2.22.0,<3.0.0']

entry_points = \
{'console_scripts': ['clone-dashboard-and-queries = '
                     'redash_toolbelt.examples.clone_dashboard_and_queries:main',
                     'export-queries = '
                     'redash_toolbelt.examples.query_export:main',
                     'find-tables = '
                     'redash_toolbelt.examples.find_table_names:main',
                     'gdpr-scrub = redash_toolbelt.examples.gdpr_scrub:lookup']}

setup_kwargs = {
    'name': 'redash-toolbelt',
    'version': '0.1.4',
    'description': 'Redash API client and tools to manage your instance.',
    'long_description': None,
    'author': 'Redash Maintainers',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import requests


class Redash(object):
    def __init__(self, redash_url, api_key):
        self.redash_url = redash_url
        self.session = requests.Session()
        self.session.headers.update({"Authorization": "Key {}".format(api_key)})

    def test_credentials(self):
        try:
            response = self._get("api/session")
            return True
        except requests.exceptions.HTTPError:
            return False

    def queries(self, page=1, page_size=25):
        """GET api/queries"""
        return self._get(
            "api/queries", params=dict(page=page, page_size=page_size)
        ).json()

    def dashboards(self, page=1, page_size=25):
        """GET api/dashboards"""
        return self._get(
            "api/dashboards", params=dict(page=page, page_size=page_size)
        ).json()

    def dashboard(self, slug):
        """GET api/dashboards/{slug}"""
        return self._get("api/dashboards/{}".format(slug)).json()

    def create_dashboard(self, name):
        return self._post("api/dashboards", json={"name": name}).json()

    def update_dashboard(self, dashboard_id, properties):
        return self._post(
            "api/dashboards/{}".format(dashboard_id), json=properties
        ).json()

    def create_widget(self, dashboard_id, visualization_id, text, options):
        data = {
            "dashboard_id": dashboard_id,
            "visualization_id": visualization_id,
            "text": text,
            "options": options,
            "width": 1,
        }
        return self._post("api/widgets", json=data)

    def duplicate_dashboard(self, slug, new_name=None):
        current_dashboard = self.dashboard(slug)

        if new_name is None:
            new_name = "Copy of: {}".format(current_dashboard["name"])

        new_dashboard = self.create_dashboard(new_name)
        if current_dashboard["tags"]:
            self.update_dashboard(
                new_dashboard["id"], {"tags": current_dashboard["tags"]}
            )

        for widget in current_dashboard["widgets"]:
            visualization_id = None
            if "visualization" in widget:
                visualization_id = widget["visualization"]["id"]
            self.create_widget(
                new_dashboard["id"], visualization_id, widget["text"], widget["options"]
            )

        return new_dashboard

    def duplicate_query(self, query_id, new_name=None):

        response = self._post(f"api/queries/{query_id}/fork")
        new_query = response.json()

        if not new_name:
            return new_query

        new_query["name"] = new_name

        return self.update_query(new_query.get("id"), new_query).json()

    def scheduled_queries(self):
        """Loads all queries and returns only the scheduled ones."""
        queries = self.paginate(self.queries)
        return filter(lambda query: query["schedule"] is not None, queries)

    def update_query(self, query_id, data):
        """POST /api/queries/{query_id} with the provided data object."""
        path = "api/queries/{}".format(query_id)
        return self._post(path, json=data)

    def paginate(self, resource):
        """Load all items of a paginated resource"""
        stop_loading = False
        page = 1
        page_size = 100

        items = []

        while not stop_loading:
            response = resource(page=page, page_size=page_size)

            items += response["results"]
            page += 1

            stop_loading = response["page"] * response["page_size"] >= response["count"]

        return items

    def _get(self, path, **kwargs):
        return self._request("GET", path, **kwargs)

    def _post(self, path, **kwargs):
        return self._request("POST", path, **kwargs)

    def _request(self, method, path, **kwargs):
        url = "{}/{}".format(self.redash_url, path)
        response = self.session.request(method, url, **kwargs)
        response.raise_for_status()
        return response
